@@ -1,6 +1,6 @@
 from .pyboard import Pyboard, PyboardError
 
-import logging, os
+import logging, time, os
 
 logger = logging.getLogger(os.path.splitext(os.path.basename(__file__))[0])
 
@@ -24,6 +24,34 @@ class Pydevice(Pyboard):
         self.use_raw_paste = device.use_raw_paste
         self.serial = device
 
+    def read_until(self, min_num_bytes, ending, timeout=10, data_consumer=None):
+        # patch: don't call data_consumer on single chars (if more data is available)
+        #        kernal print is very slow on Pi
+        # if data_consumer is used then data is not accumulated and the ending must be 1 byte long
+        assert data_consumer is None or len(ending) == 1
+        data = self.serial.read(min_num_bytes)
+        if data_consumer:
+            data_consumer(data)
+        timeout_count = 0
+        while True:
+            if data.endswith(ending):
+                if data_consumer:
+                    data_consumer(data)
+                break
+            if self.serial.inWaiting() > 0:
+                new_data = self.serial.read(1)
+                data = data + new_data
+                timeout_count = 0
+            else:
+                if data_consumer and len(data) > 0:
+                    data_consumer(data)
+                    data = b''
+                timeout_count += 1
+                if timeout is not None and timeout_count >= 100 * timeout:
+                    break
+                time.sleep(0.01)
+        return data
+
     def enter_raw_repl(self):
         self.serial.write(b"\r\x03\x03")  # ctrl-C twice: interrupt any running program
 
